#!/usr/bin/env python
# Copyright (c) 2002 by Intevation GmbH
# Authors:
# Jan-Oliver Wagner <jan@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with Thuban for details.

catchment = {}
version_str = 'greater-pre-processing 0.1'
usage_str = 'greater-pre-processing [check|help|run]'

def catchment_desc():
	# check for file "catchment.desc" and if found
	# check also for correct syntax and completeness.
	# provide appropriate error messages
	try:
		catchment_desc = open('catchment.desc', 'r').readlines()
	except:
		print "Error: Could not find or open file 'catchment.desc'"
		print "This file is required for pre-processing the catcment."
		return 1

	print "Found 'catchment.desc':"
	for line in catchment_desc:
		if line[0] == '#': continue
		line = line.rstrip()
		fields = line.split('=')
		catchment[fields[0]] = fields[1]

	for k in [ 'CATCHMENTID', 'DESCRIPTION', 'VERSION' ]:
		if not catchment.has_key(k):
			print "Error in catchment.desc: missing", k
			return 2

	print ' CatchmentId    =', catchment['CATCHMENTID']
	print ' Description    =', catchment['DESCRIPTION']
	print ' Format version =', catchment['VERSION']

	print "description complete."


def do_check():
	# check for files .drn, .rna, .dsd, .cbp, .bgd and if found
	# check also for correct syntax and completeness.
	# provide appropriate error messages
	print "Running check ..."

	if catchment['VERSION'] != '1.0':
		print ' Error: Format version ' + catchment['VERSION'] + ' can not be processed'
		print ' This check is for version 1.0'
		return 99

	# <catchmentid>.drn: digital river network
	fname = catchment['CATCHMENTID']+'.drn'
	try:
		catchment_drn = open(fname, 'r').readlines()
	except:
		print " Error: Could not find or open file '" + fname + "'"
		print " This file is required for pre-processing the catcment."
		return 1
	else:
		print ' Found ' + fname + ' (digital river network):'
	# coarse syntax check
	drn_count = 0
	line_count = 0
	for line in catchment_drn:
		line_count = line_count + 1
		if line[0] == '#': continue
		line = line.rstrip()
		fields = line.split(',')
		if len(fields) == 1:
			try:
				int(fields[0])
			except:
				print '  Syntax error at line ' + str(line_count) + ':'
				print '   StretchID ' + fields[0] + ' not an integer.'
				return 2
			drn_count = drn_count + 1
		if len(fields) == 2:
			try:
				float(fields[0])
				float(fields[1])
			except:
				print '  Syntax error at line ' + str(line_count) + ':'
				print '   coordinates (' + fields[0] + ',' + fields[1] + ') not a float tupel.'
				return 3
		if len(fields) > 2:
			print '  Syntax error at line ' + str(line_count) + ':'
			print '   incorrect number of fields (is ' + str(len(fields)) + ', should be 1 (StretchID) or 2 (coords)).'
			return 4

	print '  Total number of stretches: ' + str(drn_count)
	print '  Syntax OK'

	# <catchmentid>.cbp: catchment boundary polygon
	fname = catchment['CATCHMENTID']+'.cbp'
	try:
		catchment_cbp = open(fname, 'r').readlines()
	except:
		print "Error: Could not find or open file '" + fname + "'"
		print "This file is required for pre-processing the catcment."
		return 1
	else:
		print ' Found ' + fname + ' (catchment boundary polygon):'
	# coarse syntax check
	cbp_count = 0
	line_count = 0
	cbp_first_coord = []
	cbp_last_coord = []
	for line in catchment_cbp:
		line_count = line_count + 1
		if line[0] == '#': continue
		line = line.rstrip()
		fields = line.split(',')
		if len(fields) == 2:
			try:
				float(fields[0])
				float(fields[1])
			except:
				print '  Syntax error at line ' + str(line_count) + ':'
				print '   coordinates (' + fields[0] + ',' + fields[1] + ') not a float tupel.'
				return 3
			cbp_count = cbp_count + 1
			if cbp_count == 1:
				cbp_first_coord = fields
			cbp_last_coord = fields
		if len(fields) != 2:
			print '  Syntax error at line ' + str(line_count) + ':'
			print '   incorrect number of fields (is ' + str(len(fields)) + ', should be or 2 (coordibate pair)).'
			return 4

	if cbp_first_coord != cbp_last_coord:
		print '  Syntax error at line ' + str(line_count) + ':'
		print '   first and last coordinate pair must be equal (polygon must be closed).'
		print '   ' + str(cbp_first_coord) + ' != ' + str(cbp_last_coord)
		return 5

	print '  Total number of boundary coords: ' + str(cbp_count)
	print '  Syntax OK'

	# <catchmentid>.rna: river network attributes
	fname = catchment['CATCHMENTID']+'.rna'
	try:
		catchment_rna = open(fname, 'r').readlines()
	except:
		print "Error: Could not find or open file '" + fname + "'"
		print "This file is required for pre-processing the catcment."
		return 1
	else:
		print ' Found ' + fname + ' (river network attributes):'
	# coarse syntax check
	rna_count = 0
	line_count = 0
	for line in catchment_rna:
		line_count = line_count + 1
		if line[0] == '#': continue
		line = line.rstrip()
		fields = line.split(',')
		if len(fields) == 9:
			try:
				int(fields[0])
				float(fields[1])
				float(fields[2])
				if fields[3] != '': float(fields[3])
				if fields[4] != '': float(fields[4])
				float(fields[5])
				if fields[6] != '': float(fields[6])
				if fields[7] != '': float(fields[7])
			except:
				print '  Syntax error at line ' + str(line_count) + ':'
				print '   incorrect data type (should be int,float,float,float,float,float,float,float,string.'
				return 3
			rna_count = rna_count + 1
		if len(fields) != 9:
			print '  Syntax error at line ' + str(line_count) + ':'
			print '   incorrect number of fields (is ' + str(len(fields)) + ', should be 9).'
			return 4

	print '  Total number of stretches: ' + str(rna_count)
	print '  Syntax OK'

	# <catchmentid>.dsd: discharge site data
	fname = catchment['CATCHMENTID']+'.dsd'
	try:
		catchment_dsd = open(fname, 'r').readlines()
	except:
		print "Error: Could not find or open file '" + fname + "'"
		print "This file is required for pre-processing the catcment."
		return 1
	else:
		print ' Found ' + fname + ' (discharge site data):'
	# coarse syntax check
	dsd_count = 0
	line_count = 0
	for line in catchment_dsd:
		line_count = line_count + 1
		if line[0] == '#': continue
		line = line.rstrip()
		fields = line.split(',')
		if len(fields) == 9:
			try:
				int(fields[0])
				float(fields[1])
				float(fields[2])
				int(fields[3])
				float(fields[4])
				float(fields[5])
				int(fields[7])
			except:
				print '  Syntax error at line ' + str(line_count) + ':'
				print '   incorrect data type (should be int,float,float,int,float,float,string,int,string).'
				return 3
			dsd_count = dsd_count + 1
		if len(fields) != 9:
			print '  Syntax error at line ' + str(line_count) + ':'
			print '   incorrect number of fields (is ' + str(len(fields)) + ', should be 9).'
			return 4
		if fields[6] not in [ 'AS', 'TF' ]:
			print '  Syntax error at line ' + str(line_count) + ':'
			print "   Type is '" + fields[6] + "' but must be either 'AS' or 'TF'."
			return 6

	print '  Total number of discharge sites: ' + str(dsd_count)
	print '  Syntax OK'

	# <catchmentid>.bgd: background data
	fname = catchment['CATCHMENTID']+'.bgd'
	try:
		catchment_bgd = open(fname, 'r').readlines()
	except:
		print "Error: Could not find or open file '" + fname + "'"
		print "This file is required for pre-processing the catcment."
		return 1
	else:
		print ' Found ' + fname + ' (background data):'
	# coarse syntax check
	bgd_count = 0
	line_count = 0
	for line in catchment_bgd:
		line_count = line_count + 1
		if line[0] == '#': continue
		line = line.rstrip()
		fields = line.split(',')
		if len(fields) == 5:
			bgd_count = bgd_count + 1
		if len(fields) != 5:
			print '  Syntax error at line ' + str(line_count) + ':'
			print '   incorrect number of fields (is ' + str(len(fields)) + ', should be 5).'
			return 4
		if fields[2] not in [ '', 'point', 'polygon', 'line', 'shape', 'image', 'grid' ]:
			print '  Syntax error at line ' + str(line_count) + ':'
			print "   Type is '" + fields[2] + "' but must be one of 'point', 'polygon', 'line', 'shape', 'image', 'grid' or left empty."
			return 6
		if fields[4] not in [ 'yes', 'no' ]:
			print '  Syntax error at line ' + str(line_count) + ':'
			print "   Ctch_fl is '" + fields[4] + "' but must be either 'yes' or 'no'."
			return 6

	print '  Total number of background elements : ' + str(bgd_count)
	print '  Syntax OK'

	print 'Check on syntax successfully complete'
	return 0

def do_run():
	# execute the pre-processing
	if do_check() == 0:
		print 'Running pre-processing ...'
		print ' [not yet implemented]'
		print 'Pre-processing successfully finished'
		return 0
	else:
		print 'Not executing pre-processing due to error'

def usage():
	print usage_str
	print ' help  print this text'
	print ' check test for completeness and correct syntax of source files'
	print ' run   execute catchment pre-processing'



if __name__ == '__main__':
	import sys

	print version_str
	print

	if catchment_desc():
		sys.exit(1)

	print

	if len(sys.argv) == 2:
		if sys.argv[1] == 'check':
			do_check()
		if sys.argv[1] == 'run':
			do_run()
		if sys.argv[1] == 'help':
			usage()
	else:
		print 'usage:', usage_str
